﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AvalonRFZero.Libs
{
    static class StaticMath8Stars
    {
        public const int BoardSize = 10; // Now board width and length = 10.

        public static DateTime globalStart = new DateTime(0001, 1, 1, 0, 0, 0);
        public enum gameResult { WhiteWins, BlackWins, Draw }

        public const byte WhiteTurn = 1;
        public const byte BlackTurn = 2;
        public const byte GameOver = 100;

        public const byte EmptyFig = 0;
        public const byte WhiteFig = 1;
        public const byte BlackFig = 2;

        public const int EmptyFigStartBoard = 84; // Empty figures for standart initial position.


        public const int PosArrayLenth = (BoardSize * BoardSize) + 1;
        public const int MoveTurnIndex = BoardSize * BoardSize;
        public const int VectorMovesCountIndex = 0;

        static readonly int[,] LineVectors = { { 1, 1 }, { 0, 1 }, { -1, 1 }, { -1, 0 },
            { -1, -1 }, { 0, -1}, { 1, -1 } , { 1, 0 } };

        static readonly int[,] protectedSquares = new int[,] { { 0, 0 }, { 0, 1 }, {0, 8}, {0, 9}, { 1, 0}, { 1, 9 },
                { 8, 0 }, { 8, 9 }, { 9, 0 }, { 9, 1 }, {9, 8}, {9, 9}};

        // DRL const
        public const int nOutputsValNet = 3;
        public const int nOutputsPolNet = BoardSize * BoardSize;

        // **************  Basic functions 
        public static bool TrySquareMove(byte[] Position, int Row, int Col, int FigValue)
        {
            if (GetPosSquare(Position, Row, Col) != EmptyFig)
                return false;

            if (PointIsProtectedSquare(Row, Col))
                return false;

            for (int i = 0; i < LineVectors.GetLength(0); i++) // LineVectors strictly has 8 rows, so LineVectors.GetLength(0) is not needed.
            {
                int DeltaRow = LineVectors[i, 0];
                int DeltaCol = LineVectors[i, 1];

                int pointsToReverce = ScanVector(Position, Row, Col, FigValue, DeltaRow, DeltaCol);

                if (pointsToReverce > 0)
                    return true;
            }

            return false;

            bool PointIsProtectedSquare(int Row, int Col) 
            {
                for (int i = 0; i < protectedSquares.GetLength(0); i++) 
                {
                    if (Row == protectedSquares[i, 0] && Col == protectedSquares[i, 1])
                        return true;
                }

                return false;
            }
        }

        public static void MakeSquareMove(byte[] Position, int Row, int Col, byte FigValue)
        {
            SetPosSquare(Position, Row, Col, FigValue);

            for (int i = 0; i < 8; i++) // LineVectors strictly has 8 rows, so LineVectors.GetLength(0) is not needed.
            {
                int DeltaRow = LineVectors[i, 0];
                int DeltaCol = LineVectors[i, 1];

                int pointsToReverce = ScanVector(Position, Row, Col, FigValue, DeltaRow, DeltaCol);

                // Reverce points on the Position here...
                for (int pointCount = 1; pointCount <= pointsToReverce; pointCount++)
                {
                    SetPosSquare(Position, Row + pointCount * DeltaRow, Col + pointCount * DeltaCol, FigValue);
                }
            }

            ChangeMoveTurn(Position, FigValue);
        }

        static void ChangeMoveTurn(byte[] Position, int FigValue)
        {
            byte SameColor = 0, OppositeColor = 0;

            switch (FigValue)
            {
                case WhiteFig: { SameColor = WhiteFig; OppositeColor = BlackFig; } break;
                case BlackFig: { SameColor = BlackFig; OppositeColor = WhiteFig; } break;
            }

            // Change position move turn as well/
            // Meed to check if switch color move actually exists.
            
            Position[MoveTurnIndex] = OppositeColor;
            
            if (AnyMovesExists(Position) == true)
            {
                return; // Usual situation, when move turn is changed with Oppisite color.
            }

            else // We try here to find any moves if move turm is noy changed.
            {
                Position[MoveTurnIndex] = SameColor;
                
                if (AnyMovesExists(Position) == true)
                {
                    return; // Rare situation, when move turn is not changed with Oppisite color.
                }
                
                else
                {
                    Position[MoveTurnIndex] = GameOver; // For this position Game OVER!.
                    return;
                }
            }
        }

        static int ScanVector(byte[] Position, int moveRow, int moveCol, int FigValue, int DeltaRow, int DeltaCol)
        {
            int PointsToReverce = 0; // For now, now Position points to reverce color to FigValue

            int SameColor = 0, OppositeColor = 0;

            switch (FigValue)
            {
                case WhiteFig: { SameColor = WhiteFig; OppositeColor = BlackFig; } break;
                case BlackFig: { SameColor = BlackFig; OppositeColor = WhiteFig; } break;
            }

            // Start vector scan here....
            for (int i = 1; i <= BoardSize; i++) // Max points 8 (not 7 !!!) should be scanned in one direction with fixed DeltaRow and DeltaCol
            {
                int nextRow = moveRow + i * DeltaRow;
                int nextCol = moveCol + i * DeltaCol;

                if (SquareIsInBoard(nextRow, nextCol) == false)
                {
                    PointsToReverce = 0; break;
                }

                int squareValue = GetPosSquare(Position, nextRow, nextCol);

                if (squareValue == EmptyFig)
                {
                    PointsToReverce = 0; break;
                }

                if (squareValue == OppositeColor)
                {
                    PointsToReverce++; continue;
                }

                if (squareValue == SameColor)
                {
                    break;
                }
            }

            return PointsToReverce;
        }

        public static void SetPosSquare(byte[] Position, int Row, int Col, byte FigValue)
        {
            Position[IndxRowCol(Row, Col)] = FigValue;
        }

        public static int GetPosSquare(byte[] Position, int Row, int Col)
        {
            return Position[IndxRowCol(Row, Col)];
        }

        public static (int row, int col) RowColFromIndx(int posArrayIndex)
        {
            int row = posArrayIndex / BoardSize;
            int col = posArrayIndex - (row * BoardSize);

            return (row: row, col: col);
        }

        static bool SquareIsInBoard(int row, int col)
        {
            if (row >= 0 && row < BoardSize && col >= 0 && col < BoardSize)
                return true;
            else
                return false;
        }


        public static int IndxRowCol(int row, int col)
        {
            return (row * BoardSize + col);
        }

        // **************  Basic functions 
        // *******************************************************************************************************


        // More functions
        // Checks if at least ONE move is availavle for the Position

        public static int GetSlowRandomSeed()
        {
            Task.Delay(25).Wait(); // Wait some time, to generate UNIQUE ID based on timer.
            Random Random = new Random();
            int rndSeed = Random.Next(1000000);
            return rndSeed;
        }

        public static byte[] CreateRandomStartPosition(Random rnd, int rndMovesTotal)
        {
            int rndMovesNow = FindFlexRandomValue();

            byte[] gamePosition = CreateStandartInitialPosition();

            for (int moveNext = 0; moveNext < rndMovesNow; moveNext++)
            {
                int[] rndMove = TakeRandomMove(rnd, gamePosition);

                MakeSquareMove(gamePosition, rndMove[0], rndMove[1], gamePosition[MoveTurnIndex]);
            }

            return gamePosition;

            int FindFlexRandomValue()
            {
                if (rndMovesTotal == 0)
                    return 0;

                double segmentLenght = 1 / (double)rndMovesTotal;

                double[] segPoints = new double[rndMovesTotal + 1];

                for (int i = 0; i < segPoints.Length; i++)
                {
                    if (i == 0)
                        segPoints[i] = 0;
                    else if (i == segPoints.Length - 1)
                        segPoints[i] = 1;
                    else
                    {
                        segPoints[i] = segmentLenght * i;
                    }
                }

                double rndVal = rnd.NextDouble();

                for (int i = 0; i < segPoints.Length; i++)
                {
                    if (rndVal > segPoints[i] && rndVal < segPoints[i + 1])
                        return (i + 1);
                }

                return 0;
            }
        }

        public static int[] TakeRandomMove(Random rnd, byte[] Position)
        {
            List<int[]> allMovesList = GetAllMoves(Position);
            int rndMoveNumber = rnd.Next(0, allMovesList.Count);

            int moveRow = allMovesList[rndMoveNumber][0];
            int moveCol = allMovesList[rndMoveNumber][1];

            return new int[] { moveRow, moveCol };
        }

        public static List<int[]> GetAllMoves(byte[] Position) 
        {
            List<int[]> movesAvailable = new List<int[]>();

            for (int row = 0; row < BoardSize; row++) 
            {
                for (int col = 0; col < BoardSize; col++) 
                {
                    if (TrySquareMove(Position, row, col, Position[MoveTurnIndex]) == true) 
                    {
                        movesAvailable.Add(new int[2] { row, col });
                    }
                }
            }

            return movesAvailable;
        }

        public static List<List<int[]>> GetAllMovesForListOfPositions(List<byte[]> positionList) 
        {
            List<List<int[]>> fullMovesList = new List<List<int[]>>(positionList.Count);

            for (int i = 0; i < positionList.Count; i++) 
            {
                List<int[]> nextMoves = GetAllMoves(Position: positionList[i]);
                fullMovesList.Add(nextMoves);
            }

            return fullMovesList;
        }

        public static byte[] CopyPosition(byte[] sourcePosition)
        {
            byte[] copyPosition = new byte[PosArrayLenth];
            Array.Copy(sourcePosition, copyPosition, PosArrayLenth);
            return copyPosition;
        }

        public static byte[] GetNextPosition(byte[] Position, int moveRow, int moveCol)
        {
            byte[] nextPosition = new byte[PosArrayLenth];
            Array.Copy(Position, nextPosition, PosArrayLenth);
            
            MakeSquareMove(nextPosition, moveRow, moveCol, Position[MoveTurnIndex]);
            return nextPosition;
        }

        public static List<byte[]> GetAllNextPositions(byte[] Position, List<int[]> movesList) 
        {
            List<byte[]> nextPositions = new List<byte[]>(movesList.Count);

            for (int i = 0; i < movesList.Count; i++)
            {
                byte[] nextPosition = GetNextPosition(Position: Position, moveRow: movesList[i][0], moveCol: movesList[i][1]);
                nextPositions.Add(nextPosition);
            }

            return nextPositions;
        }

        public static (int WhiteCount, int BlackCount) StaticResult(byte[] Position)
        {
            int WhiteCount = 0;
            int BlackCount = 0;
            int EmptyFig = 0;

            for (int row = 0; row < BoardSize; row++)
            {
                for (int col = 0; col < BoardSize; col++)
                {
                    switch (GetPosSquare(Position, row, col))
                    {
                        case WhiteFig: WhiteCount++; break;
                        case BlackFig: BlackCount++; break;
                        
                        default: EmptyFig++; break;
                    }
                }
            }

            return (WhiteCount: WhiteCount, BlackCount: BlackCount);
        }

        public static int EmptyFigCount(byte[] Position)
        {
            int EmptyFigCount = 0;
            
            for (int row = 0; row < BoardSize; row++)
            {
                for (int col = 0; col < BoardSize; col++)
                {
                    switch (GetPosSquare(Position, row, col))
                    {
                        case EmptyFig: EmptyFigCount++; break;
                        
                        default: break;
                    }
                }
            }

            return EmptyFigCount - protectedSquares.GetLength(0);
        }

        public static int FiguresOnTheBoard(byte[] Position) 
        {
            int FiguresOnTheBoard = 0;

            for (int row = 0; row < BoardSize; row++)
            {
                for (int col = 0; col < BoardSize; col++)
                {
                    switch (GetPosSquare(Position, row, col))
                    {
                        case WhiteFig: FiguresOnTheBoard++; break;
                        case BlackFig: FiguresOnTheBoard++; break;

                        default: break;
                    }
                }
            }

            return FiguresOnTheBoard;
        }

        static bool AnyMovesExists(byte[] Position)
        {
            for (int row = 0; row < BoardSize; row++)
            {
                for (int col = 0; col < BoardSize; col++)
                {
                    if (TrySquareMove(Position, row, col, Position[MoveTurnIndex]) == true)
                    {
                        return true; // at least ONE move found.
                    }
                }
            }

            return false; // No moves found here....
        }

        public static byte[] CreateStandartInitialPosition()
        {
            byte[] Position = new byte[PosArrayLenth];
            Position[MoveTurnIndex] = BlackTurn;

            SetPosSquare(Position, 5, 4, WhiteFig);
            SetPosSquare(Position, 5, 5, BlackFig);
            SetPosSquare(Position, 4, 4, BlackFig);
            SetPosSquare(Position, 4, 5, WhiteFig);

            return Position;
        }

        public static byte[] PosStringToByteArray(string posString)
        {
            string[] partsString = posString.Split('.');
            byte[] PosArray = new byte[partsString.Length];

            for (int i = 0; i < partsString.Length; i++)
            {
                PosArray[i] = byte.Parse(partsString[i]);
            }

            return PosArray;
        }

        public static string PosArrayToString(byte[] Position)
        {
            return string.Join('.', Position);
        }

        public static int[] GetRandomSortedArray(Random rnd, int arrSize)
        {
            List<IntSort> ListToSort = new List<IntSort>(arrSize);

            for (int i = 0; i < arrSize; i++)
            {
                ListToSort.Add(new IntSort(i, rnd.NextDouble()));
            }

            IntSort.NNOrder RndOrder = new IntSort.NNOrder();
            ListToSort.Sort(RndOrder);

            int[] randomIntArr = new int[arrSize];

            for (int i = 0; i < arrSize; i++)
            {
                randomIntArr[i] = ListToSort[i].IntIndex;
            }

            return randomIntArr;
        }

        public static List<int[]> MiniBatchSplit(int NMiniBatches, int ArrayLength)
        {
            List<int[]> batchSplit = new List<int[]>(NMiniBatches);

            int singleBatchObjects;

            if (ArrayLength % NMiniBatches == 0)
            {
                singleBatchObjects = (int)(ArrayLength / NMiniBatches);
            }
            else
            {
                singleBatchObjects = (int)(ArrayLength / NMiniBatches) + 1;
            }


            for (int i = 0; i < NMiniBatches; i++)
            {
                batchSplit.Add(new int[2]);

                batchSplit[i][0] = singleBatchObjects * i;

                if (i != NMiniBatches - 1)
                {
                    batchSplit[i][1] = batchSplit[i][0] + singleBatchObjects - 1;
                }

                else
                {
                    batchSplit[i][1] = ArrayLength - 1;
                }
            }

            return batchSplit;
        }
    }
}
