using System;
using Avalonia.Controls;
using Avalonia.Interactivity;
using Tmds.DBus.Protocol;
using AvalonRFZero.PRG;
using AvalonRFZero.Libs;
using System.Data;
using System.Collections.Generic;
using SkiaSharp;
using System.Collections.ObjectModel;
using Microsoft.VisualBasic;
using Avalonia.Styling;
using Avalonia.Media;
using Avalonia.Controls.Shapes;

namespace AvalonRFZero
{
    public partial class MainWindow : Window
    {
        GBoard boardMain;

        BoardMoves boardMoves;

        // DataTable2 boardTable2;

        internal HumanComputerMatch HCMatch = null;

        // ObservableCollection<Person> people2 = new ObservableCollection<Person>();

        Random rndMain = new Random();
        public MainWindow()
        {
            InitializeComponent();

            // Set moves object
            boardMoves = new BoardMoves(MovesGrid1);
            // boardMoves.FormatTable();

            // int DataGridColumns = MovesGrid1.Columns.Count;
        }

        public void Window_Loaded(object sender, RoutedEventArgs e)
        {
            this.WindowState = WindowState.Maximized;
            this.Canvas01.SizeChanged += Canvas01_SizeChanged;
            
            // Draw main board
            boardMain = new GBoard(Canvas01, moveTurn);

            this.Canvas01.PointerPressed += Canvas01_PointerPressed;

            // Set moves object
            // boardMoves = new BoardMoves(MovesGrid1);

            // Set data 2
            // boardTable2 = new DataTable2(MovesGrid02);

            //people2.Add(new Person("", ""));
            //people2.Add(new Person("", ""));

            // Canvas01.PointerPressed 

            AppStart.SetStartParams(this);

            // SetDataGridRowFontSize(fontSize: 40);

            int DataGridColumns = MovesGrid1.Columns.Count;

            boardMoves.FormatTable(MovesGrid1);
        }

        private void Canvas01_PointerPressed(object sender, Avalonia.Input.PointerPressedEventArgs e)
        {
            var point = e.GetCurrentPoint(sender as Control);
            double  mouseX = point.Position.X;
            double  mouseY = point.Position.Y;

            (int row, int col) = boardMain.GetRowColFromMouseCoordinates(mouseX, mouseY);

            if (HCMatch != null)
                HCMatch.MakeHumanrMove(row, col);
        }

        public void StartHCGames_Click(object sender, RoutedEventArgs args) 
        {
            string player01 = cbHCEngine01.Items[cbHCEngine01.SelectedIndex].ToString();
            string player02 = cbHCEngine02.Items[cbHCEngine02.SelectedIndex].ToString();

            lbProgHCResult.IsVisible = false;
            
            lbGameProgressInfo.IsVisible = true;
            lbGameProgressInfo.Content = $"White: {player01}, Black: {player02}";

            HCMatch = new HumanComputerMatch(boardMain, boardMoves, lbProgHCResult,
                cbHCEngine01.SelectedIndex, cbHCEngine02.SelectedIndex);
        }

        public void StartEngineGames_Click(object sender, RoutedEventArgs args)
        {
            EngineMatch engineMatch = new EngineMatch();

            engineMatch.StartEngineMatchAsinc(gBoard: boardMain,
                movesVisualiser: boardMoves,
                gamesTotal: Convert.ToInt32(tbGamesTotal.Text),
                lbEngine1MoveInfo: lbEngine1MoveInfo, lbEngine2MoveInfo: lbEngine2MoveInfo,
                labelGameResult: lbGameProgressInfo,
                labelMatchResult: lbMathResult, labelEngine1: lbEngine1Info,
                labelEngine2: lbEngine2Info, 
                Engine01Index: cbEngine01.SelectedIndex, Engine02Index: cbEngine02.SelectedIndex,
                startButton: btStartSelfPlay, RndMoves: 0);
        }

        private void Canvas01_SizeChanged(object? sender, SizeChangedEventArgs e)
        {
            boardMain.DrawBoard();
        }

        public void Draw_Click(object sender, RoutedEventArgs args)
        {
            boardMain.DrawBoard();
        }

        public void Draw_PositionClick(object sender, RoutedEventArgs args)
        {
            int rndMovesTotal = rndMain.Next(0, 20);
            byte[] newPosition = StaticMath8Stars.CreateRandomStartPosition(rnd: rndMain, rndMovesTotal: rndMovesTotal);
            
            boardMain.DrawPosition(newPosition);
        }

        public void AddMoveBlack_Click(object sender, RoutedEventArgs args)
        {
            boardMoves.AddMoveToTable(move: new int[2] { 3, 5 }, moveturn: StaticMath8Stars.BlackTurn);
        }

        public void AddMoveWhite_Click(object sender, RoutedEventArgs args)
        {
            boardMoves.AddMoveToTable(move: new int[2] { 6, 6 }, moveturn: StaticMath8Stars.WhiteTurn);
        }

        public void ClearLastRow_Click(object sender, RoutedEventArgs args)
        {
            boardMoves.ClearLastRow();
            Brush brushRec = new SolidColorBrush(Colors.LightGreen);

            recRed.Fill = brushRec;
        }

        public void ClearTable_Click(object sender, RoutedEventArgs args)
        {
            boardMoves.ClearTable();
        }

        public void BtClear_Click(object sender, RoutedEventArgs args)
        {
            boardMain.ClearBoard();
        }

        public void ShowAbout_Click(object sender, RoutedEventArgs args) 
        {
            // int DataGridColumns = MovesGrid1.Columns.Count;

            var window = new WinAbout();
            window.Show();
        }

        public class Person
        {
            public string FirstName { get; set; }
            public string LastName { get; set; }

            public Person(string firstName, string lastName)
            {
                FirstName = firstName;
                LastName = lastName;
            }
        }
    }
}