﻿using Avalonia.Controls;
using Avalonia.Controls.Primitives;
using Avalonia.Layout;
using Avalonia.Media;
using Avalonia.Styling;
using AvalonRFZero.Libs;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static AvalonRFZero.MainWindow;
using static AvalonRFZero.PRG.BoardMoves;

namespace AvalonRFZero.PRG
{
    internal class BoardMoves
    {
        // public ObservableCollection<Person> peopleCollection = new ObservableCollection<Person>();

        public ObservableCollection<RowMoves> moveCollection = new ObservableCollection<RowMoves>();

        // List<Person> people = new List<Person>();

        DataGrid dataGridMoves;
        public BoardMoves(DataGrid dataGrid) 
        {
            this.dataGridMoves = dataGrid;

            dataGridMoves.ItemsSource = moveCollection;
        }


        public void FormatTable(DataGrid dataGrid)
        {
            // Assuming MovesGrid1 is already defined in your XAML or code-behind
            var column01 = dataGrid.Columns.FirstOrDefault(c => c.Header.ToString() == "MoveBlack") as DataGridTextColumn;
            var column02 = dataGrid.Columns.FirstOrDefault(c => c.Header.ToString() == "MoveWhite") as DataGridTextColumn;

            column01.FontSize = 20;
            column02.FontSize = 20;

            double colWidth = (dataGrid.Bounds.Width / 2) - 1;

            column01.Width = new DataGridLength(colWidth);
            column02.Width = new DataGridLength(colWidth);

            // Create a new Style to target DataGridColumnHeader elements
            Style headerStyle = new Style(x => x.OfType<DataGridColumnHeader>());

            // Set the Background color to Red
            headerStyle.Setters.Add(new Setter(DataGridColumnHeader.BackgroundProperty, Brushes.Red));

            // Set the Font Size to 20
            headerStyle.Setters.Add(new Setter(DataGridColumnHeader.FontSizeProperty, 20.0));

            // Set the Font Color to Yellow
            headerStyle.Setters.Add(new Setter(DataGridColumnHeader.ForegroundProperty, Brushes.Yellow));


            // Add the Style to the DataGrid's Styles collection
            dataGrid.Styles.Add(headerStyle);

            // Style for DataGridColumnHeaders (to center header text)
            Style headerTextStyle = new Style(x => x.OfType<DataGridColumnHeader>());
            headerTextStyle.Setters.Add(new Setter(DataGridColumnHeader.HorizontalContentAlignmentProperty, HorizontalAlignment.Center));
            dataGrid.Styles.Add(headerTextStyle);

            // Style for DataGridCells (to center cell content text)
            Style cellTextStyle = new Style(x => x.OfType<DataGridCell>());
            cellTextStyle.Setters.Add(new Setter(DataGridCell.HorizontalContentAlignmentProperty, HorizontalAlignment.Center));
            dataGrid.Styles.Add(cellTextStyle);

        }

        public void ClearTable()
        {
            moveCollection.Clear();
        }

        public void ClearLastRow()
        {
            if (moveCollection.Count > 0) 
                moveCollection.Remove(moveCollection.Last());
        }


        public void AddMoveToTable(int[] move, int moveturn) 
        {
            if (move == null) // Start position here.....
            {
                ClearTable();
                return;
            }

            if (moveturn == StaticMath8Stars.BlackTurn) // Black turn is here.... , we always add new row if turn is BLACK.
            {
                RowMoves rowMoves = new RowMoves();

                rowMoves.MoveBlack = MoveToNotation(move);

                moveCollection.Add(rowMoves);
            }

            else // White turn is here.... , need check if we need to add new row or just use current last row 
            {
                // sometimes we need to add move for white then first move is for White,
                // for example when we heve random generated position with first move for White
                
                if (moveCollection.Count == 0) 
                {
                    RowMoves rowMoves = new RowMoves();

                    rowMoves.MoveWhite = MoveToNotation(move);

                    moveCollection.Add(rowMoves);
                    return;
                }

                RowMoves lastRow = moveCollection.Last();

                if (lastRow.MoveBlack != null && lastRow.MoveWhite == null) // Move Black exists, move white does not exists.
                {
                    string moveBlack = lastRow.MoveBlack;

                    ClearLastRow(); // We need to remove last row anyway, and rebuild it again.

                    RowMoves rowMoves = new RowMoves();
                    
                    rowMoves.MoveBlack = moveBlack;
                    rowMoves.MoveWhite = MoveToNotation(move);

                    moveCollection.Add(rowMoves);

                }

                else  // Move Black does not exists, move white does not exists.
                { 
                    RowMoves rowMoves = new RowMoves();

                    rowMoves.MoveWhite = MoveToNotation(move);

                    moveCollection.Add(rowMoves); 
                    
                } 
            }

        }

        public static string MoveToNotation(int[] move)
        {
            int moveRow = move[0];
            int moveCol = move[1];

            string vertLinename = "";

            switch (moveCol)
            {
                case 0: vertLinename = "A"; break;
                case 1: vertLinename = "B"; break;
                case 2: vertLinename = "C"; break;
                case 3: vertLinename = "D"; break;
                case 4: vertLinename = "E"; break;
                case 5: vertLinename = "F"; break;
                case 6: vertLinename = "G"; break;
                case 7: vertLinename = "H"; break;
                case 8: vertLinename = "I"; break;
                case 9: vertLinename = "J"; break;
            }

            return $"{vertLinename}{moveRow + 1}";
        }

        public class RowMoves
        {
            public string MoveBlack { get; set; } = null;
            public string MoveWhite { get; set; } = null;
        }

    }
}
