﻿using Avalonia.Controls;
using AvalonRFZero.Libs;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using static AvalonRFZero.PRG.EngineMatch;

namespace AvalonRFZero.PRG
{
    internal class EngineMatch
    {
        const string EngineMatchSetupFile = @"SETUP\EngineMatchSetup.txt";

        int Engine1Wins = 0;
        int Engine2Wins = 0;
        int DrawsCount = 0;

        GBoard gBoard;

        Label lbEngine1MoveInfo;
        Label lbEngine2MoveInfo;
        Label labelGameResult;
        Label labelMatchResult;

        Button startButton;

        BoardMoves movesVisualiser;

        public async void StartEngineMatchAsinc(GBoard gBoard, BoardMoves movesVisualiser, int gamesTotal,
             Label lbEngine1MoveInfo, Label lbEngine2MoveInfo, Label labelGameResult,
             Label labelMatchResult, Label labelEngine1,
             Label labelEngine2, int Engine01Index, int Engine02Index, 
             Button startButton, int RndMoves)
        {

            this.gBoard = gBoard;
            this.lbEngine1MoveInfo = lbEngine1MoveInfo;
            this.lbEngine2MoveInfo = lbEngine2MoveInfo;
            this.labelGameResult = labelGameResult;
            this.labelMatchResult = labelMatchResult;
            this.startButton = startButton;
            this.movesVisualiser = movesVisualiser;

            Progress<PositionReport> progressPosition = new Progress<PositionReport>();
            progressPosition.ProgressChanged += ProgressPosition_ProgressChanged; ;

            Progress<MatchReport> progressMatchReport = new Progress<MatchReport>();
            progressMatchReport.ProgressChanged += ProgressMatchReport_ProgressChanged; ;

            Progress<GameStatReport> progressStartReport = new Progress<GameStatReport>();
            progressStartReport.ProgressChanged += ProgressStartReport_ProgressChanged; ;

            // (IEngine Engine1, IEngine Engine2) = EngineSerial.CreateDuelEnginesFromFile(DuelSetupFile: @"Setup\DuelEngineSetup.txt");

            (IEngine Engine1, IEngine Engine2) = SetEngineByIndex(Engine01Index, Engine02Index);

            labelEngine1.Content = $"Engine1: {Engine1.ShowEngineInfo()}";
            labelEngine2.Content = $"Engine2: {Engine2.ShowEngineInfo()}";

            this.labelMatchResult.Content = $"{0} - {0}";

            // startButton.Enabled = false;

            await Task.Run(() => StartEngineMatch(progressMatchReport: progressMatchReport, progressPosition: progressPosition,
                    progressStartReport: progressStartReport, Engine1: Engine1, Engine2: Engine2, gamesTotal: gamesTotal, RndMoves: RndMoves));

            // startButton.Enabled = true;
        }

        private void ProgressStartReport_ProgressChanged(object sender, GameStatReport e)
        {
            labelGameResult.Content = $"Plaing game: {e.GameIndex} of {e.GamesTotal} - White: {e.EngineWhite.engineName}, Black: {e.EngineBlack.engineName}";
        }

        private void ProgressMatchReport_ProgressChanged(object sender, MatchReport e)
        {
            string gameRes;

            if (e.LastGameResult == StaticMath8Stars.gameResult.WhiteWins)
                gameRes = "White wins:";
            else if (e.LastGameResult == StaticMath8Stars.gameResult.BlackWins)
                gameRes = "Black wins:";
            else
                gameRes = "Draw:";

            labelGameResult.Content = $"{gameRes} - {e.LastGameWhiteCount} : " +
                $"{e.LastGameBlackCount}!";

            this.labelMatchResult.Content = $"{e.Engine1Wins + e.DrawsCount * 0.5f} - {e.Engine2Wins + e.DrawsCount * 0.5f}";
        }

        private void ProgressPosition_ProgressChanged(object sender, PositionReport e)
        {
            gBoard.DrawPosition(e.PositionCurrent);

            if (e.LastMoveEngineIndex == 1) 
            {
                // Need show stat here...
                lbEngine1MoveInfo.Content = $"Engine 1: {e.MoveInfo}";
            }

            if (e.LastMoveEngineIndex == 2)
            {
                // Need show stat here...
                lbEngine2MoveInfo.Content = $"Engine 2: {e.MoveInfo}";
            }
            
            movesVisualiser.AddMoveToTable(move: e.LastMove, moveturn: e.LastMoveTurn);

        }

        public void StartEngineMatch(IProgress<MatchReport> progressMatchReport, IProgress<PositionReport> progressPosition,
            IProgress<GameStatReport> progressStartReport, IEngine Engine1, IEngine Engine2, int gamesTotal, int RndMoves)
        {
            int gamesPlayed = 0;

            for (int gIndex = 0; gIndex < gamesTotal; gIndex++)
            {
                // byte[] Position = StaticMath03.CreateStandartInitialPosition(StaticMath03.BlackTurn);

                // Create random position here...
                byte[] rndStartPosition = StaticMath8Stars.CreateRandomStartPosition(rnd: new Random(), rndMovesTotal: RndMoves);

                IEngine EngineWhite, EngineBlack;

                if (gIndex % 2 == 0) // games 0, 2, 4 etc.
                {
                    EngineWhite = Engine1;
                    EngineBlack = Engine2;
                }
                else // games 1, 3, 5 etc.
                {
                    EngineWhite = Engine2;
                    EngineBlack = Engine1;
                }

                progressStartReport.Report(new GameStatReport(EngineWhite: EngineWhite, EngineBlack: EngineBlack,
                    GameIndex: gIndex, GamesTotal: gamesTotal));

                (StaticMath8Stars.gameResult Result, int WhiteCount, int BlackCount) =
                    TakeSingleGame(progressPosition: progressPosition, Position: rndStartPosition,
                    Engine1: Engine1, Engine2: Engine2,
                    EngineWhite: EngineWhite, EngineBlack: EngineBlack,
                    NeedRecordPositions: true);

                UpdateGlobalResults(Engine1: Engine1, EngineWhite: EngineWhite, EngineBlack: EngineBlack, GameResult: Result);

                progressMatchReport.Report(new MatchReport(Engine1: Engine1, Engine2: Engine2,
                    EngineWhite: EngineWhite, EngineBlack: EngineBlack,
                    Engine1Wins: Engine1Wins, Engine2Wins: Engine2Wins, DrawsCount: DrawsCount,
                    LastGameResult: Result, LastGameWhiteCount: WhiteCount, LastGameBlackCount: BlackCount, LastGameIndex: gIndex + 1
                    ));

                gamesPlayed++;

                Task.Delay(2000).Wait();
            }
        }

        static (StaticMath8Stars.gameResult Result, int WhiteCount, int BlackCount) TakeSingleGame(IProgress<PositionReport> progressPosition,
             byte[] Position, IEngine Engine1, IEngine Engine2, IEngine EngineWhite, IEngine EngineBlack, bool NeedRecordPositions)
        {
            // Report start position here...
            progressPosition.Report(new PositionReport(Position: Position, LastMoveMade: null, LastMoveTurn: -1, LastMoveEngineIndex: 0,
                MoveInfo: " -- "));

            Task.Delay(1000).Wait(); // Wait 1000 milseconds before next. 

            while (Position[StaticMath8Stars.MoveTurnIndex] != StaticMath8Stars.GameOver)
            {
                BestMoveData bmData;
                IEngine EngineLastMadeMove;

                if (Position[StaticMath8Stars.MoveTurnIndex] == StaticMath8Stars.WhiteFig)
                {
                    bmData = EngineWhite.ShowsBestMoveAndStat(Position);
                    EngineLastMadeMove = EngineWhite;
                }

                else
                {
                    bmData = EngineBlack.ShowsBestMoveAndStat(Position);
                    EngineLastMadeMove = EngineBlack;
                }

                // We are making board move here....
                int moveTurn = Position[StaticMath8Stars.MoveTurnIndex];

                StaticMath8Stars.MakeSquareMove(Position, bmData.BestMove[0], bmData.BestMove[1], Position[StaticMath8Stars.MoveTurnIndex]);
                
                // We need to check if last nove made by Engine1 or Engine2
                int LastMoveEngineIndex = 0;
                
                if (EngineLastMadeMove == Engine1)
                    LastMoveEngineIndex = 1;
                else
                    LastMoveEngineIndex = 2;

                progressPosition.Report(new PositionReport(Position, bmData.BestMove, moveTurn, LastMoveEngineIndex: LastMoveEngineIndex,
                    MoveInfo: bmData.MoveInfo));

                Task.Delay(500).Wait(); // Wait 500 milseconds before next move. 
            }

            (StaticMath8Stars.gameResult GameResult, int WhiteCount, int BlackCount) = CalcGameResult(Position);
            return (Result: GameResult, WhiteCount: WhiteCount, BlackCount: BlackCount);
        }

        static public (StaticMath8Stars.gameResult GameResult, int WhiteCount, int BlackCount) CalcGameResult(byte[] finalPosition)
        {
            (int whiteCount, int blackCount) = StaticMath8Stars.StaticResult(finalPosition);

            if (whiteCount - blackCount > 0) // White wins
            {
                return (GameResult: StaticMath8Stars.gameResult.WhiteWins, WhiteCount: whiteCount, BlackCount: blackCount);
            }
            else if (whiteCount - blackCount < 0) // Black wins
            {
                return (GameResult: StaticMath8Stars.gameResult.BlackWins, WhiteCount: whiteCount, BlackCount: blackCount);
            }
            else // Draw.
            {
                return (GameResult: StaticMath8Stars.gameResult.Draw, WhiteCount: whiteCount, BlackCount: blackCount);
            }
        }

        void UpdateGlobalResults(IEngine Engine1, IEngine EngineWhite, IEngine EngineBlack, StaticMath8Stars.gameResult GameResult)
        {
            switch (GameResult)
            {
                case StaticMath8Stars.gameResult.WhiteWins:
                    {
                        if (EngineWhite == Engine1)
                            this.Engine1Wins++;
                        else
                            this.Engine2Wins++;
                    }
                    break;

                case StaticMath8Stars.gameResult.BlackWins:
                    {
                        if (EngineBlack == Engine1)
                            this.Engine1Wins++;
                        else
                            this.Engine2Wins++;
                    }
                    break;

                case StaticMath8Stars.gameResult.Draw:
                    this.DrawsCount++;
                    break;
            }
        }

        (IEngine Engine1, IEngine Engine2) SetEngineByIndex(int Engine01Index, int Engine02Index)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(EngineMatchSetupFile);

            XmlNode nodeEngineRoot = doc.DocumentElement.SelectSingleNode("/Engines");

            int currIndex = 0;

            IEngine Engine1 = null;
            IEngine Engine2 = null;

            foreach (XmlNode childNode in nodeEngineRoot.ChildNodes)
            {
                if (currIndex == Engine01Index)
                {
                    Engine1 = EngineSerial.CreateEngineFromXmlNode(nodeEngine: childNode);
                }

                if (currIndex == Engine02Index)
                {
                    Engine2 = EngineSerial.CreateEngineFromXmlNode(nodeEngine: childNode);
                }

                currIndex++;
            }

            return (Engine1: Engine1, Engine2: Engine2);
        }


        public static void CreateEngineList(ComboBox player1, ComboBox player2)
        {
            player1.Items.Clear();
            player2.Items.Clear();

            XmlDocument doc = new XmlDocument();
            doc.Load(EngineMatchSetupFile);

            XmlNode nodeEngineRoot = doc.DocumentElement.SelectSingleNode("/Engines");

            foreach (XmlNode childNode in nodeEngineRoot.ChildNodes)
            {
                player1.Items.Add(childNode.Attributes["EngineName"].InnerText);
                player2.Items.Add(childNode.Attributes["EngineName"].InnerText);
            }

            player1.SelectedIndex = 0;
            player2.SelectedIndex = 0;
        }

        public class GameStatReport
        {
            public IEngine EngineWhite { set; get; }
            public IEngine EngineBlack { set; get; }
            public int GameIndex { set; get; }
            public int GamesTotal { set; get; }

            public GameStatReport(IEngine EngineWhite, IEngine EngineBlack, int GameIndex, int GamesTotal)
            {
                this.EngineWhite = EngineWhite;
                this.EngineBlack = EngineBlack;

                this.GameIndex = GameIndex + 1;
                this.GamesTotal = GamesTotal;
            }
        }

        public class PositionReport
        {
            public byte[] PositionCurrent { set; get; } = new byte[StaticMath8Stars.PosArrayLenth];
            public int[] LastMove { set; get; } = new int[2];
            public int LastMoveTurn { set; get; }
            public int LastMoveEngineIndex { set; get; }
            public string MoveInfo { set; get; }

            public PositionReport(byte[] Position, int[] LastMoveMade, int LastMoveTurn, int LastMoveEngineIndex, string MoveInfo)
            {
                Array.Copy(Position, PositionCurrent, Position.Length);

                if (LastMoveMade != null)
                {
                    Array.Copy(LastMoveMade, LastMove, LastMoveMade.Length);
                    this.LastMoveTurn = LastMoveTurn;
                }
                else
                {
                    LastMove = null;
                    this.LastMoveTurn = -1;
                }

                this.LastMoveEngineIndex = LastMoveEngineIndex;
                this.MoveInfo = MoveInfo;
            }
        }

        public class MatchReport
        {
            public IEngine Engine1 { set; get; }
            public IEngine Engine2 { set; get; }

            public IEngine EngineWhite { set; get; }
            public IEngine EngineBlack { set; get; }

            public int Engine1Wins { set; get; }
            public int Engine2Wins { set; get; }
            public int DrawsCount { set; get; }

            public StaticMath8Stars.gameResult LastGameResult { set; get; }
            public int LastGameWhiteCount { set; get; }
            public int LastGameBlackCount { set; get; }
            public int LastGameIndex { set; get; }

            public MatchReport(IEngine Engine1, IEngine Engine2, IEngine EngineWhite, IEngine EngineBlack,
                int Engine1Wins, int Engine2Wins, int DrawsCount,
                StaticMath8Stars.gameResult LastGameResult, int LastGameWhiteCount, 
                int LastGameBlackCount, int LastGameIndex)
            {
                this.Engine1 = Engine1;
                this.Engine2 = Engine2;
                this.EngineWhite = EngineWhite;
                this.EngineBlack = EngineBlack;
                this.Engine1Wins = Engine1Wins;
                this.Engine2Wins = Engine2Wins;
                this.DrawsCount = DrawsCount;
                this.LastGameResult = LastGameResult;
                this.LastGameWhiteCount = LastGameWhiteCount;
                this.LastGameBlackCount = LastGameBlackCount;
                this.LastGameIndex = LastGameIndex;
            }
        }
    }
}
