﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using AvalonRFZero.Engines;

namespace AvalonRFZero.PRG
{
    internal static class EngineSerial
    {
        public static IEngine CreateEngineFromXmlNode(XmlNode nodeEngine, int SqlEpoch = -1, bool RoundRobin = false)
        {
            IEngine Engine = null;

            switch (nodeEngine.Attributes["EngineType"].InnerText)
            {
                case "EngineRandom": Engine = CreateEngineRandom(nodeEngine); break;
                case "RFZeroX": Engine = CreateEngineRFZeroX(nodeEngine, SqlEpoch, RoundRobin); break;
            }

            return Engine;
        }

        public static IEngine CreateEngineRandom(XmlNode nodeEngine)
        {
            string engineName = nodeEngine.Attributes["EngineName"].InnerText;
            string engineParams = $"Type: Random";
            IEngine Engine = new EngineRandom(engineName, engineParams);

            return Engine;
        }

        public static IEngine CreateEngineRFZeroX(XmlNode nodeEngine, int SqlEpoch, bool RoundRobin)
        {
            int mctsLoopsMax = Convert.ToInt32(nodeEngine.Attributes["MctsLoopsMax"].InnerText);
            int nodeLoops = Convert.ToInt32(nodeEngine.Attributes["NodeLoops"].InnerText);
            float expParam = Convert.ToSingle(nodeEngine.Attributes["ExpParam"].InnerText);

            string NNVersion = nodeEngine.Attributes["NNVersion"].InnerText;
            string NNFileName = nodeEngine.Attributes["NNFileName"].InnerText;

            // bool GPU = (nodeEngine.Attributes["GPU"].InnerText == "1") ? true : false;

            string engineParams = $"Type: RFZeroX, MCTS loops: {mctsLoopsMax} Node loops: {nodeLoops} expParam {expParam}";
            string engineName;
            string netFilePath;

            if (SqlEpoch == -1)
            {
                engineName = nodeEngine.Attributes["EngineName"].InnerText;
                netFilePath = $"{AppConst.savedNetFilePath}{NNFileName}";
            }

            else
            {
                if (RoundRobin == false)
                {
                    engineName = nodeEngine.Attributes["EngineName"].InnerText;
                    netFilePath = AppConst.NNForMining(SqlEpoch - 1); // If mining games for epoch 2 we need to use SQL Net for epoch 1, etc.
                }
                else
                {
                    engineName = AppConst.EngineNameForSQLTournament(SqlEpoch);
                    netFilePath = AppConst.NNForSqlTournament(SqlEpoch);
                }
            }

            IEngine Engine = new RFZeroX(engineName: engineName,
                                      NNVersion: NNVersion,
                                      engineParams: engineParams,
                                      mctsLoopsMax: mctsLoopsMax,
                                      nodeLoops: nodeLoops,
                                      netFilePath: netFilePath,
                                      expParam: expParam);
            return Engine;
        }













































    }
}
